﻿/*
    Program demonstrující použití lokální funkce uvnitř iterátoru a ošetření výjimky.
*/

using System;
using System.Collections.Generic;

namespace LocalFunction
{
    class Program
    {
        public static IEnumerable<int> OddSequence1(int start, int end)
        {
            if (start < 0 || start > 99)
                throw new ArgumentOutOfRangeException
                    ("Počáteční hodnota musí být v rozmezí 0 až 99");
            if (end > 100)
                throw new ArgumentOutOfRangeException
                    ("Koncová hodnota musí být menší nebo rovna 100");
            if (start >= end)
                throw new ArgumentException
                    ("Počáteční hodnota musí být menší než koncová");

            for (int i = start; i <= end; i++)
            {
                if (i % 2 == 1)
                    yield return i;
            }
        }

        public static IEnumerable<int> OddSequence2(int start, int end)
        {
            if (start < 0 || start > 99)
                throw new ArgumentOutOfRangeException
                    ("Počáteční hodnota musí být v rozmezí 0 až 99");
            if (end > 100)
                throw new ArgumentOutOfRangeException
                    ("Koncová hodnota musí být menší nebo rovna 100");
            if (start >= end)
                throw new ArgumentException
                    ("Počáteční hodnota musí být menší než koncová");

            return GetOddSequenceEnumerator();

            IEnumerable<int> GetOddSequenceEnumerator()
            {
                for (int i = start; i <= end; i++)
                {
                    if (i % 2 == 1)
                        yield return i;
                }
            }
        }

        static void Main()
        {
            try
            {
                IEnumerable<int> ienum1 = OddSequence1(50, 110);
                Console.WriteLine("Vrácený enumerátor 1:");
                foreach (var i in ienum1)
                {
                    Console.Write("{0,4}", i);
                }
            }
            catch (ArgumentException ex)
            {
                Console.WriteLine("Nastala výjimka: " + ex.Message);
            }

            Console.WriteLine();

            try
            {
                IEnumerable<int> ienum2 = OddSequence2(50, 110);
                Console.WriteLine("Vrácený enumerátor 2:");
                foreach (var i in ienum2)
                {
                    Console.Write("{0,4}", i);
                }
            }
            catch (ArgumentException ex)
            {
                Console.WriteLine("Nastala výjimka: " + ex.Message);
            }
        }
    }
}

